@@ -299,8 +299,11 @@ def get_the_rest(density, densityAll, multiplicityAll, cdf=None, cdfAll=None):
 
     k = list(f1 == max(f1)).index(True)
     multiplicityRest[k:] = multiplicityRightTail[k:]
+    
+    # Force positivity
+    cdfRestMax = np.maximum.accumulate(cdfRest)
 
-    return cdfRest, multiplicityRest
+    return cdfRestMax, multiplicityRest
 
 
 def expected_payoff(density, densityAll, multiplicityAll, cdf=None, cdfAll=None):
@@ -525,7 +528,7 @@ def implicit_state_prices(density, densityAll, multiplicityAll=None, cdf=None, c
     implicit = list()
     for k in offsets:
         if k == int(k):
-            offset_cdf = integer_shift(cdf, k)
+            offset_cdf = integer_shift(cdf, int(k))
             ip = expected_payoff(density=None, densityAll=densityAll, multiplicityAll=multiplicityAll, cdf=offset_cdf,
                                  cdfAll=cdfAll)
             implicit.append(np.sum(ip))
